 //*************************************************************************************************//
// Titolo: 			 			Controllo otto servi v1.0.0
//					 			(tratto da un lavoro di Samuel Oliver Wane 	http://www.fcet.staffs.ac.uk/sow1)
//					
// Autore:			 			Marcello Pinna	09/06/2013
// 
// Descrizione: 	 			Controllo e pilotaggio di uno o pi (max 8) servicomando tramite interfaccia grafica (GUI)
// 
// Hardware:		 			Freedom I (by Mauro Laurenti - www.Laurtec.com)
//					 			pic 18F4550
//					 			quarzo 20 MHz
//					 			servicomando Futaba S3003
//
// Software di programmazione:	MPLABX v1.51 + compilatore C18 v3.40
//
// Note importanti:  			i servi devono essere collegati alla PORTD del pic 18F4550 ed alimentati con una tensione
//					 			continua di 6 volts. Il loro controllo e pilotaggio, ad eccezione della routine del loop,
//					 			 compatibile con il programma RS232 Terminal (vers. 1.3.1 o superiore), sviluppato da
//					 			Mauro Laurenti (www.Laurtec.com), secondo i seguenti parametri di comunicazione
//					 			Impostazioni RS232 Terminal v1.3.1:
//  				 			Port Setting: COM1, 19200, 8, 1, none, none
//					 			Write setting -> Data format : Integer
//					 			Display -> Data format : Integer
//					 			Sliders -> Data format -> Send Slider Index & Enable: Entrambi spuntati			
//*************************************************************************************************//

#include <p18F4550.h>
#include <usart.h>

#define FACTOR 500				// fattore di conversione mS - conteggi

#pragma config FOSC = HS 		// OSC    = HS   Impostato per lavorare ad alta frequenza 	
#pragma config WDT = OFF 		// WDT    = OFF	 Disabilito il watchdog timer
#pragma config LVP = OFF 		// LVP    = OFF  Disabilito programmazione LVP	
#pragma config PBADEN = OFF		// PBADEN = OFF  Disabilito gli ingressi analogici

// ============================= VARIABILI DI CONTROLLO ==================================
	
unsigned int timer0; 							// vedi timers.h
unsigned int timer1; 							// vedi timers.h

unsigned int counter;							// contatore generico
unsigned int index;								// indice (numero) del servo utilizzato
unsigned int value;								// valore in uscita dallo slider		
unsigned int pos[] = {15,15,15,15,15,15,15,15};	// inizializzazione dell'array: ogni servo  in posizione centrale
unsigned int ms;								// variabile che assume ogni valore contenuto nell'array							
	
// ============================= PROTOTIPI DI FUNZIONE ===================================

void WriteTimer0 ( unsigned int timer0); // Prototipo di funzione estrapolato dal timers.h (MCC18\h)
void WriteTimer1 ( unsigned int timer1); // Prototipo di funzione estrapolato dal timers.h (MCC18\h)
void High_Int_Event (void);			 	 // Prototipo di funzione per alta priorita'

// ======================== IMPOSTAZIONI TIMER0 E TIMER1 =================================

void set_timer(void) {

	// ===== TIMER0 =====
	// Imposto la modalit a 16 bit
	T0CONbits.T08BIT = 0;
	
	// Imposto il clock interno
	T0CONbits.T0CS = 0;

	// Disabilito Prescaler 
	T0CONbits.PSA = 1;

	// Abilito le interruzioni del Timer0
	INTCONbits.TMR0IE = 1;

	// Abilito le interruzioni del Timer0 ad alta priorit
	INTCON2bits.TMR0IP = 1;

	// Abilito il Timer0
	T0CONbits.TMR0ON = 1;
		
	// ===== TIMER1 =====
	// Imposto la modalit a 16 bit
	T1CONbits.RD16 = 1;
		
	// Abilito il clock interno Timer1  (Fosc/4)
	T1CONbits.TMR1CS = 0; 
	
	// Imposto il PreScaler Timer1 1:2
	T1CONbits.T1CKPS0 = 1;
	T1CONbits.T1CKPS1 = 0;
	
	// Disabilito l'oscillatore del Timer1 
	T1CONbits.T1OSCEN = 0;
	
	// Abilito il Timer1
	T1CONbits.TMR1ON = 1;
	
	// CALCOLO DEL VALORE INIZIALE DEL TIMER1...
	// Voglio generare un impulso della durata totale di 20 ms; con il prescaler impostato 1:2
	// il tempo di incremento  di 0.0004 ms. Il numero dei conteggi  dato da (20/0.0004 = 50000)
	// Sottraendo a 65536 il numero dei conteggi cos ottenuti si ha: (65536-50000 = 15536)
	timer1 = 15536;
	
	// Imposto la funzione WriteTimer1 con questo valore
	WriteTimer1 (timer1);
		
	// Abilito l'interrupt ad alta priorit
	INTCONbits.GIEH = 1; 
}

// =========================== ROUTINE INTERRUPT ALTA PRIORITA' =========================
 
#pragma code High_vector=0x08

void High_interrupt (void) {
	
	// Imposta il salto per la gestione dell'interrupt ad alta priorit
	_asm GOTO High_Int_Event _endasm 
}

#pragma code

#pragma interrupt High_Int_Event

// Funzione per la gestione dell'interruzione ad alta priorita'
void High_Int_Event (void) {

	// Appena scatta l'interrupt del Timer0
	if (INTCONbits.TMR0IF) {

	// Spengo i servi
	LATD = 0;
	
	// Disabilito il Timer0
	T0CONbits.TMR0ON = 0; 
	}
	
	// Appena scatta l'interrupt del Timer1
	if (PIR1bits.TMR1IF) {
	
	// Ricarico il Timer1
	WriteTimer1(timer1);
	
	// Azzero il flag di interrupt del Timer1
    PIR1bits.TMR1IF = 0;
	
	// In funzione dell'indice del servo, assegno alla variabile ms il valore contenuto nella corrispondente 
	// cella dell'array, quindi imposto la porta D per muoverlo nella posizione desiderata. 
	switch (index) {
	
	case (1): LATD = 0x01; ms = pos[index-1]; break;	
	case (2): LATD = 0x02; ms = pos[index-1]; break;	
	case (3): LATD = 0x04; ms = pos[index-1]; break;	
	case (4): LATD = 0x08; ms = pos[index-1]; break;	
	case (5): LATD = 0x10; ms = pos[index-1]; break;	
	case (6): LATD = 0x20; ms = pos[index-1]; break;	
	case (7): LATD = 0x40; ms = pos[index-1]; break;	
	case (8): LATD = 0x80; ms = pos[index-1]; break;	
	}
				
	// Converto i ms del servo azionato in numero di conteggi da levare a 65536 per ottenere il valore
	// iniziale del Timer0
	timer0 = (unsigned int)(65536 - (ms * FACTOR));
	
	// Imposto la funzione WriteTimer0 con questo valore
	WriteTimer0 (timer0);

	// Abilito il Timer0
    T0CONbits.TMR0ON = 1;
	
	// Azzero il flag di interrupt del Timer0
	INTCONbits.TMR0IF = 0;
 	}
}

// ==================================== ROUTINE PRINCIPALE ==============================

void main (void) {

	// Imposto TRISA tutti ingressi 
	LATA = 0x00; 
 	TRISA = 0xFF;

	// Imposto TRISB tutti ingressi
 	LATB = 0x00; 
 	TRISB = 0xFF;

	// Imposto TRISC tutti ingressi
 	LATC = 0x00;    	 
 	TRISC = 0xFF;
 	
	// Imposto TRISD tutte uscite per pilotare i servi (servo1 -> RD0 ... servo8 -> RD7)
 	LATD = 0x00;  
 	TRISD = 0x00;

	// Imposto TRISE tutti ingressi
 	LATE = 0x00;
 	TRISE = 0xFF;

	// Configura l'USART
 	// 8 bit
 	// 19200 bit/s
 	// 1 bit stop
 	// 0 bit parit
 	OpenUSART( USART_TX_INT_OFF &
 	USART_RX_INT_OFF &
 	USART_ASYNCH_MODE &
 	USART_EIGHT_BIT &
 	USART_CONT_RX &
 	USART_BRGH_HIGH,
 	64 );
	
	// Avvio la routine di inizializzazione
	set_timer();

	// Ciclo infinito
 	while(1) {
	
	// Attendo di ricevere "index" dal PC
 	while(!DataRdyUSART( ));

 	// Leggo "index" dal buffer di ricezione 
 	index = ReadUSART();
	
	// Attendo di ricevere "value" dal PC
 	while(!DataRdyUSART( ));

 	// Leggo "value" dal buffer di ricezione 
 	value = ReadUSART();
	
	// In funzione dell'indice del servo aggiorno la corrispondente cella dell'array
	pos[index-1] = value;
				
	// Invio "index" al terminale
 	WriteUSART (index);
	
	// Attendo che "index" venga trasmesso
 	while (BusyUSART());
					
	// Invio "value" al terminale
 	WriteUSART (value);
	
	// Attendo che "value" venga trasmesso
 	while (BusyUSART());
	
 	} // fine while
}

